<?php

namespace Teknisa\CADFIN\Services;

use Teknisa\CADFIN\Factories\Ger08500ContaCorrenteFactory;
use Teknisa\Model\Fiscal\Entities\Contcorr;
use Teknisa\CADFIN\Services\FluxoDeCaixaService;
use Teknisa\Libs\Util\DateUtil;
use Doctrine\ORM\EntityManager;

class Ger08500ContaCorrenteService{

    protected $factory;
    protected $environment;
    protected $fluxodecaixaservice;

    public function __construct(Ger08500ContaCorrenteFactory $factory, $environment, FluxoDeCaixaService $fluxodecaixaservice, EntityManager $entityManager) {
        $this->factory = $factory;
        $this->environment = $environment;
        $this->fluxodecaixaservice = $fluxodecaixaservice;
        $this->entityManager = $entityManager;
    }

    public function validarContaCorrente($row){
        $contcorr = $this->factory->getContcorr($row["CDREDUCONT"],$this->environment->getNrOrgTrab());
        if($contcorr != null){
            throw new \Exception("Impossível selecionar. Registro já existe.");
        }
    }

    public function implantaSaldoContaCorrente($row){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try{
            $this->validaSaldoImplantado($row);
            $dtsdoccorr = $this->formataData($row['DTSDOCCORR']);
            
            $sdoccorr = $this->factory->insereSaldoContaCorrente($dtsdoccorr, $row['CDREDUCONT'], $row['VRSDOCCORR'], $row['VRSDOIMPLA'], $this->environment->getCdoperador(), $this->environment->getNrOrgTrab());
            $this->entityManager->persist($sdoccorr);
            $this->entityManager->flush();
            $this->callProcedureEvolucao($row['CDREDUCONT'], $row['DTSDOCCORR'], 0, 0);
            $this->entityManager->flush();
            $connection->commit();
        }catch (\Exception $e) {
            $connection->rollback();
            throw $e;
        }
    }

    public function validaSaldoImplantado($row){
        $dtsdoccorr = $this->formataData($row['DTSDOCCORR']);
        $existeSaldo = $this->factory->getSaldoContaCorrente($row['CDREDUCONT'], $dtsdoccorr, $this->environment->getNrOrgTrab());
        if ($existeSaldo){
            throw new \Exception("Data da implantação do saldo de conta corrente já cadastrada.");
        }
    }

    public function excluiImplantacaoSaldo($row){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try{
            $dtsdoccorr = $this->formataData($row['DTSDOCCORR']);
            $sdoccorr = $this->factory->getSaldoContaCorrente($row['CDREDUCONT'], $dtsdoccorr, $this->environment->getNrOrgTrab());
            $this->entityManager->remove($sdoccorr);
            $this->entityManager->flush();
            $this->callProcedureEvolucao($row['CDREDUCONT'], $dtsdoccorr->format('d/m/Y'), 0, 0);
            $this->entityManager->flush();
            $connection->commit();
        }catch (\Exception $e) {
            $connection->rollback();
            throw $e;
        }
    }

    public function evolucaoSaldo($rows){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try{
            foreach($rows as $row){
                $this->callProcedureEvolucao($row['CDREDUCONT'], $row['DATAINICIAL'], 0, 0);
            }
            $this->entityManager->flush();
            $connection->commit();
        }catch (\Exception $e) {
            $connection->rollback();
            throw $e;
        }
    }

    public function callProcedureEvolucao($cdreducont, $datainicial, $vrsdoccorr, $idatualiza){
        $this->factory->execProcAtualizaSaldoBancario($cdreducont, $datainicial, $vrsdoccorr, $idatualiza);
    }

    public function formataData($data){
        if(strlen($data) == 19)
            $data = DateUtil::truncateString($data);
        $data = DateUtil::getDatadeString($data);
        $data = DateUtil::truncateData($data);

        return $data;
    }

}